#include <xsi_application.h>
#include <xsi_context.h>
#include <xsi_pluginregistrar.h>
#include <xsi_status.h>

#include <xsi_icenodecontext.h>
#include <xsi_icenodedef.h>
#include <xsi_command.h>
#include <xsi_factory.h>
#include <xsi_longarray.h>
#include <xsi_doublearray.h>
#include <xsi_math.h>
#include <xsi_vector2f.h>
#include <xsi_vector3.h>
#include <xsi_vector3f.h>
#include <xsi_vector4f.h>
#include <xsi_matrix3f.h>
#include <xsi_matrix4f.h>
#include <xsi_rotationf.h>
#include <xsi_quaternionf.h>
#include <xsi_color4f.h>
#include <xsi_shape.h>
#include <xsi_icegeometry.h>
#include <xsi_iceportstate.h>
#include <xsi_indexset.h>
#include <xsi_dataarray.h>
#include <xsi_dataarray2D.h>

#include <vector>

#define ADDABLES  ( siICENodeDataFloat | siICENodeDataLong | siICENodeDataVector2 | siICENodeDataVector3 | siICENodeDataVector4 | siICENodeDataColor4 | siICENodeDataQuaternion )
#define MULTABLES  ( siICENodeDataFloat | siICENodeDataLong | siICENodeDataVector3 | siICENodeDataColor4 | siICENodeDataQuaternion )

enum IDs
{
	ID_IN_ARRAY,
	ID_IN_IDCS,
	ID_IN_SIZE,
	ID_IN_INC,
	ID_G_100 = 100,
	ID_OUT_ARRAY = 200,
	ID_TYPE_CNS = 400,
	ID_STRUCT_CNS,
	ID_CTXT_CNS,
	ID_UNDEF = ULONG_MAX
};

XSI::CStatus RegisterGetGroupedSumsArray( XSI::PluginRegistrar& in_reg );
XSI::CStatus RegisterGetGroupedProductsArray( XSI::PluginRegistrar& in_reg );

using namespace XSI; 

SICALLBACK XSILoadPlugin( PluginRegistrar& in_reg )
{
	in_reg.PutAuthor(L"RCLAUS");
	in_reg.PutName(L"ICE Array Nodes C++");
	in_reg.PutVersion(1,0);

	RegisterGetGroupedSumsArray( in_reg );
	RegisterGetGroupedProductsArray( in_reg );

	return CStatus::OK;
}

SICALLBACK XSIUnloadPlugin( const PluginRegistrar& in_reg )
{
	CString strPluginName;
	strPluginName = in_reg.GetName();
	return CStatus::OK;
}

CStatus RegisterGetGroupedSumsArray( PluginRegistrar& in_reg )
{
	ICENodeDef nodeDef;
	nodeDef = Application().GetFactory().CreateICENodeDef(L"Get Grouped Sums Array",L"Get Grouped Sums Array");

	CStatus st;
	st = nodeDef.PutColor(154,188,102);
	st.AssertSucceeded( ) ;

	// IN
	st = nodeDef.AddPortGroup(ID_G_100);
	st.AssertSucceeded( ) ;

	st = nodeDef.AddInputPort(ID_IN_ARRAY, ID_G_100, ADDABLES, siICENodeStructureArray, siICENodeContextAny, L"Array", L"Array", false, CValue(), CValue(), ID_TYPE_CNS, ID_UNDEF, ID_CTXT_CNS);
	st.AssertSucceeded( ) ;

	st = nodeDef.AddInputPort(ID_IN_SIZE, ID_G_100, siICENodeDataLong,  siICENodeStructureAny, siICENodeContextAny, L"Group Size(s)", L"GroupSize", 1, 1, 16, ID_UNDEF, ID_UNDEF, ID_UNDEF);
	st.AssertSucceeded( ) ; 

	st = nodeDef.AddInputPort(ID_IN_INC, ID_G_100, siICENodeDataLong,  siICENodeStructureSingle, siICENodeContextAny, L"Increment", L"Increment", 1, 0, 16, ID_UNDEF, ID_UNDEF, ID_UNDEF);
	st.AssertSucceeded( ) ;

	// OUT
	st = nodeDef.AddOutputPort(ID_OUT_ARRAY, ADDABLES, siICENodeStructureArray, siICENodeContextAny, L"Result Array", L"ResultArray", ID_TYPE_CNS, ID_UNDEF, ID_CTXT_CNS);
	st.AssertSucceeded( ) ;

	PluginItem nodeItem = in_reg.RegisterICENode(nodeDef);
	nodeItem.PutCategories(L"Array");

	return CStatus::OK;
}

SICALLBACK GetGroupedSumsArray_Evaluate( ICENodeContext& in_ctxt )
{
	ULONG out_portID = in_ctxt.GetEvaluatedOutputPortID( );
	if(out_portID != ID_OUT_ARRAY) return CStatus::OK;

	CDataArray2DLong inSizeArray( in_ctxt, ID_IN_SIZE );
	CDataArrayLong inIncrement( in_ctxt, ID_IN_INC );		
	CIndexSet indexSet( in_ctxt );

	siICENodeDataType dt;
	siICENodeStructureType st;
	siICENodeContextType ct;
	in_ctxt.GetPortInfo(ID_OUT_ARRAY, dt, st, ct);
	switch(dt) {
		case XSI::siICENodeDataFloat: 
			{
				CDataArray2DFloat inArray( in_ctxt, ID_IN_ARRAY );	
				CDataArray2DFloat outArray( in_ctxt );	
				for(CIndexSet::Iterator it = indexSet.Begin(); it.HasNext(); it.Next())
				{
					if(!inArray.GetCount())
						continue;
					CDataArray2DFloat::Accessor inArrayA = inArray[it];
					CDataArray2DLong::Accessor inSizeA = inSizeArray[it];

					ULONG nbArray = inArrayA.GetCount();
					if(!nbArray)
						continue;

					ULONG nbGroupSizes = inSizeA.GetCount();
					ULONG group1st = 0, increment = inIncrement[it];
					ULONG nbGroups = nbGroupSizes > 1 ? nbGroupSizes : nbArray/inSizeA[0];

					CDataArray2DFloat::Accessor outArrayA = outArray.Resize( it, nbGroups );
					for(int i_g=0; i_g<nbGroups; i_g++) 
					{
						ULONG groupSize = inSizeA[nbGroupSizes>1 ? i_g : 0];
						ULONG offset = 0;
						float sum = 0;
						for(int i_ge=0; i_ge<groupSize; i_ge++)
						{
							if(group1st+offset < nbArray) 
								sum += inArrayA[group1st+offset];							
							offset += increment ? increment:1;
						}
						outArrayA[i_g] = sum;
						if(increment == 1)
							group1st += groupSize;
						else
							group1st += increment>1 ? 1:increment;
					}
				}
				break;
			}
		case XSI::siICENodeDataLong: 
		{
				CDataArray2DLong inArray( in_ctxt, ID_IN_ARRAY );	
				CDataArray2DLong outArray( in_ctxt );	
				for(CIndexSet::Iterator it = indexSet.Begin(); it.HasNext(); it.Next())
				{
					if(!inArray.GetCount())
						continue;
					CDataArray2DLong::Accessor inArrayA = inArray[it];
					CDataArray2DLong::Accessor inSizeA = inSizeArray[it];

					ULONG nbArray = inArrayA.GetCount();
					if(!nbArray)
						continue;

					ULONG nbGroupSizes = inSizeA.GetCount();
					ULONG group1st = 0, increment = inIncrement[it];
					ULONG nbGroups = nbGroupSizes > 1 ? nbGroupSizes : nbArray/inSizeA[0];

					CDataArray2DLong::Accessor outArrayA = outArray.Resize( it, nbGroups );
					for(int i_g=0; i_g<nbGroups; i_g++) 
					{
						ULONG groupSize = inSizeA[nbGroupSizes>1 ? i_g : 0];
						ULONG offset = 0;
						ULONG sum = 0;
						for(int i_ge=0; i_ge<groupSize; i_ge++)
						{
							if(group1st+offset < nbArray) 
								sum += inArrayA[group1st+offset];							
							offset += increment ? increment:1;
						}
						outArrayA[i_g] = sum;
						if(increment == 1)
							group1st += groupSize;
						else
							group1st += increment>1 ? 1:increment;
					}
				}
				break;
				}
		case XSI::siICENodeDataVector2:
				{
					CDataArray2DVector2f inArray( in_ctxt, ID_IN_ARRAY );	
					CDataArray2DVector2f outArray( in_ctxt );	
					for(CIndexSet::Iterator it = indexSet.Begin(); it.HasNext(); it.Next())
					{
						if(!inArray.GetCount())
							continue;
						CDataArray2DVector2f::Accessor inArrayA = inArray[it];
						CDataArray2DLong::Accessor inSizeA = inSizeArray[it];

						ULONG nbArray = inArrayA.GetCount();
						if(!nbArray)
							continue;

						ULONG nbGroupSizes = inSizeA.GetCount();
						ULONG group1st = 0, increment = inIncrement[it];
						ULONG nbGroups = nbGroupSizes > 1 ? nbGroupSizes : nbArray/inSizeA[0];

						CDataArray2DVector2f::Accessor outArrayA = outArray.Resize( it, nbGroups );
						for(int i_g=0; i_g<nbGroups; i_g++) 
						{
							ULONG groupSize = inSizeA[nbGroupSizes>1 ? i_g : 0];
							ULONG offset = 0;
							XSI::MATH::CVector2f sum;
							for(int i_ge=0; i_ge<groupSize; i_ge++)
							{
								if(group1st+offset < nbArray) 
									sum.Add(sum, inArrayA[group1st+offset]);							
								offset += increment ? increment:1;
							}
							outArrayA[i_g] = sum;
							if(increment == 1)
								group1st += groupSize;
							else
								group1st += increment>1 ? 1:increment;
						}
					}
					break;
				}
			case XSI::siICENodeDataVector3:
				{
					CDataArray2DVector3f inArray( in_ctxt, ID_IN_ARRAY );		
					CDataArray2DVector3f outArray( in_ctxt );	
					for(CIndexSet::Iterator it = indexSet.Begin(); it.HasNext(); it.Next())
					{
						if(!inArray.GetCount())
							continue;
						CDataArray2DVector3f::Accessor inArrayA = inArray[it];
						CDataArray2DLong::Accessor inSizeA = inSizeArray[it];

						ULONG nbArray = inArrayA.GetCount();
						if(!nbArray)
							continue;

						ULONG nbGroupSizes = inSizeA.GetCount();
						ULONG group1st = 0, increment = inIncrement[it];
						ULONG nbGroups = nbGroupSizes > 1 ? nbGroupSizes : nbArray/inSizeA[0];

						CDataArray2DVector3f::Accessor outArrayA = outArray.Resize( it, nbGroups );
						for(int i_g=0; i_g<nbGroups; i_g++) 
						{
							ULONG groupSize = inSizeA[nbGroupSizes>1 ? i_g : 0];
							ULONG offset = 0;
							XSI::MATH::CVector3f sum;
							for(int i_ge=0; i_ge<groupSize; i_ge++)
							{
								if(group1st+offset < nbArray) 
									sum.Add(sum, inArrayA[group1st+offset]);							
								offset += increment ? increment:1;
							}
							outArrayA[i_g] = sum;
							if(increment == 1)
								group1st += groupSize;
							else
								group1st += increment>1 ? 1:increment;
						}
					}
					break;
				}
				case XSI::siICENodeDataVector4:
				{
					CDataArray2DVector4f inArray( in_ctxt, ID_IN_ARRAY );	
					CDataArray2DVector4f outArray( in_ctxt );	
					for(CIndexSet::Iterator it = indexSet.Begin(); it.HasNext(); it.Next())
					{
						if(!inArray.GetCount())
							continue;
						CDataArray2DVector4f::Accessor inArrayA = inArray[it];
						CDataArray2DLong::Accessor inSizeA = inSizeArray[it];

						ULONG nbArray = inArrayA.GetCount();
						if(!nbArray)
							continue;

						ULONG nbGroupSizes = inSizeA.GetCount();
						ULONG group1st = 0, increment = inIncrement[it];
						ULONG nbGroups = nbGroupSizes > 1 ? nbGroupSizes : nbArray/inSizeA[0];

						CDataArray2DVector4f::Accessor outArrayA = outArray.Resize( it, nbGroups );
						for(int i_g=0; i_g<nbGroups; i_g++) 
						{
							ULONG groupSize = inSizeA[nbGroupSizes>1 ? i_g : 0];
							ULONG offset = 0;
							XSI::MATH::CVector4f sum;
							for(int i_ge=0; i_ge<groupSize; i_ge++)
							{
								if(group1st+offset < nbArray) 
									sum.Add(sum, inArrayA[group1st+offset]);							
								offset += increment ? increment:1;
							}
							outArrayA[i_g] = sum;
							if(increment == 1)
								group1st += groupSize;
							else
								group1st += increment>1 ? 1:increment;
						}
					}
					break;
				}
				case XSI::siICENodeDataQuaternion:
				{
					CDataArray2DQuaternionf inArray( in_ctxt, ID_IN_ARRAY );	
					CDataArray2DQuaternionf outArray( in_ctxt );	
					for(CIndexSet::Iterator it = indexSet.Begin(); it.HasNext(); it.Next())
					{
						if(!inArray.GetCount())
							continue;
						CDataArray2DQuaternionf::Accessor inArrayA = inArray[it];
						CDataArray2DLong::Accessor inSizeA = inSizeArray[it];

						ULONG nbArray = inArrayA.GetCount();
						if(!nbArray)
							continue;

						ULONG nbGroupSizes = inSizeA.GetCount();
						ULONG group1st = 0, increment = inIncrement[it];
						ULONG nbGroups = nbGroupSizes > 1 ? nbGroupSizes : nbArray/inSizeA[0];

						CDataArray2DQuaternionf::Accessor outArrayA = outArray.Resize( it, nbGroups );
						for(int i_g=0; i_g<nbGroups; i_g++) 
						{
							ULONG groupSize = inSizeA[nbGroupSizes>1 ? i_g : 0];
							ULONG offset = 0;
							XSI::MATH::CQuaternionf sum;
							for(int i_ge=0; i_ge<groupSize; i_ge++)
							{
								if(group1st+offset < nbArray) 
									sum.Add(sum, inArrayA[group1st+offset]);
								offset += increment ? increment:1;
							}
							outArrayA[i_g] = sum;
							if(increment == 1)
								group1st += groupSize;
							else
								group1st += increment>1 ? 1:increment;
						}
					}
					break;
				}
				case XSI::siICENodeDataColor4:
				{
					CDataArray2DColor4f inArray( in_ctxt, ID_IN_ARRAY );
					CDataArray2DColor4f outArray( in_ctxt );	
					for(CIndexSet::Iterator it = indexSet.Begin(); it.HasNext(); it.Next())
					{
						if(!inArray.GetCount())
							continue;
						CDataArray2DColor4f::Accessor inArrayA = inArray[it];
						CDataArray2DLong::Accessor inSizeA = inSizeArray[it];

						ULONG nbArray = inArrayA.GetCount();
						if(!nbArray)
							continue;

						ULONG nbGroupSizes = inSizeA.GetCount();
						ULONG group1st = 0, increment = inIncrement[it];
						ULONG nbGroups = nbGroupSizes > 1 ? nbGroupSizes : nbArray/inSizeA[0];

						CDataArray2DColor4f::Accessor outArrayA = outArray.Resize( it, nbGroups );
						for(int i_g=0; i_g<nbGroups; i_g++) 
						{
							ULONG groupSize = inSizeA[nbGroupSizes>1 ? i_g : 0];
							ULONG offset = 0;
							XSI::MATH::CColor4f sum;
							for(int i_ge=0; i_ge<groupSize; i_ge++)
							{
								if(group1st+offset < nbArray) 
									sum.Add(inArrayA[group1st+offset]);
								offset += increment ? increment:1;
							}
							outArrayA[i_g] = sum;
							if(increment == 1)
								group1st += groupSize;
							else
								group1st += increment>1 ? 1:increment;
						}
					}
					break;
				}
	}

	return CStatus::OK;
}

CStatus RegisterGetGroupedProductsArray( PluginRegistrar& in_reg )
{
	ICENodeDef nodeDef;
	nodeDef = Application().GetFactory().CreateICENodeDef(L"Get Grouped Products Array",L"Get Grouped Products Array");

	CStatus st;
	st = nodeDef.PutColor(154,188,102);
	st.AssertSucceeded( ) ;

	// IN
	st = nodeDef.AddPortGroup(ID_G_100);
	st.AssertSucceeded( ) ;

	st = nodeDef.AddInputPort(ID_IN_ARRAY, ID_G_100, MULTABLES, siICENodeStructureArray, siICENodeContextAny, L"Array", L"Array", false, CValue(), CValue(), ID_TYPE_CNS, ID_UNDEF, ID_CTXT_CNS);
	st.AssertSucceeded( ) ;

	st = nodeDef.AddInputPort(ID_IN_SIZE, ID_G_100, siICENodeDataLong,  siICENodeStructureAny, siICENodeContextAny, L"Group Size(s)", L"GroupSize", 1, 1, 16, ID_UNDEF, ID_UNDEF, ID_UNDEF);
	st.AssertSucceeded( ) ; 

	st = nodeDef.AddInputPort(ID_IN_INC, ID_G_100, siICENodeDataLong,  siICENodeStructureSingle, siICENodeContextAny, L"Increment", L"Increment", 1, 0, 16, ID_UNDEF, ID_UNDEF, ID_UNDEF);
	st.AssertSucceeded( ) ;

	// OUT
	st = nodeDef.AddOutputPort(ID_OUT_ARRAY, MULTABLES, siICENodeStructureArray, siICENodeContextAny, L"Result Array", L"ResultArray", ID_TYPE_CNS, ID_UNDEF, ID_CTXT_CNS);
	st.AssertSucceeded( ) ;

	PluginItem nodeItem = in_reg.RegisterICENode(nodeDef);
	nodeItem.PutCategories(L"Array");

	return CStatus::OK;
}

SICALLBACK GetGroupedProductsArray_Evaluate( ICENodeContext& in_ctxt )
{
	ULONG out_portID = in_ctxt.GetEvaluatedOutputPortID( );
	if(out_portID != ID_OUT_ARRAY) return CStatus::OK;

	CDataArray2DLong inSizeArray( in_ctxt, ID_IN_SIZE );
	CDataArrayLong inIncrement( in_ctxt, ID_IN_INC );		
	CIndexSet indexSet( in_ctxt );

	siICENodeDataType dt;
	siICENodeStructureType st;
	siICENodeContextType ct;
	in_ctxt.GetPortInfo(ID_OUT_ARRAY, dt, st, ct);
	switch(dt) {
		case XSI::siICENodeDataFloat: 
			{
				CDataArray2DFloat inArray( in_ctxt, ID_IN_ARRAY );	
				CDataArray2DFloat outArray( in_ctxt );	
				for(CIndexSet::Iterator it = indexSet.Begin(); it.HasNext(); it.Next())
				{
					if(!inArray.GetCount())
						continue;
					CDataArray2DFloat::Accessor inArrayA = inArray[it];
					CDataArray2DLong::Accessor inSizeA = inSizeArray[it];

					ULONG nbArray = inArrayA.GetCount();
					if(!nbArray)
						continue;

					ULONG nbGroupSizes = inSizeA.GetCount();
					ULONG group1st = 0, increment = inIncrement[it];
					ULONG nbGroups = nbGroupSizes > 1 ? nbGroupSizes : nbArray/inSizeA[0];

					CDataArray2DFloat::Accessor outArrayA = outArray.Resize( it, nbGroups );
					for(int i_g=0; i_g<nbGroups; i_g++) 
					{
						ULONG groupSize = inSizeA[nbGroupSizes>1 ? i_g : 0];
						ULONG offset = 0;
						float prod = 1;
						for(int i_ge=0; i_ge<groupSize; i_ge++)
						{
							if(group1st+offset < nbArray) 
								prod *= inArrayA[group1st+offset];							
							offset += increment ? increment:1;
						}
						outArrayA[i_g] = prod;
						if(increment == 1)
							group1st += groupSize;
						else
							group1st += increment>1 ? 1:increment;
					}
				}
				break;
			}
		case XSI::siICENodeDataLong: 
		{
				CDataArray2DLong inArray( in_ctxt, ID_IN_ARRAY );	
				CDataArray2DLong outArray( in_ctxt );	
				for(CIndexSet::Iterator it = indexSet.Begin(); it.HasNext(); it.Next())
				{
					if(!inArray.GetCount())
						continue;
					CDataArray2DLong::Accessor inArrayA = inArray[it];
					CDataArray2DLong::Accessor inSizeA = inSizeArray[it];

					ULONG nbArray = inArrayA.GetCount();
					if(!nbArray)
						continue;

					ULONG nbGroupSizes = inSizeA.GetCount();
					ULONG group1st = 0, increment = inIncrement[it];
					ULONG nbGroups = nbGroupSizes > 1 ? nbGroupSizes : nbArray/inSizeA[0];

					CDataArray2DLong::Accessor outArrayA = outArray.Resize( it, nbGroups );
					for(int i_g=0; i_g<nbGroups; i_g++) 
					{
						ULONG groupSize = inSizeA[nbGroupSizes>1 ? i_g : 0];
						ULONG offset = 0;
						ULONG prod = 1;
						for(int i_ge=0; i_ge<groupSize; i_ge++)
						{
							if(group1st+offset < nbArray) 
								prod *= inArrayA[group1st+offset];							
							offset += increment ? increment:1;
						}
						outArrayA[i_g] = prod;
						if(increment == 1)
							group1st += groupSize;
						else
							group1st += increment>1 ? 1:increment;
					}
				}
				break;
				}
			case XSI::siICENodeDataVector3:
				{
					CDataArray2DVector3f inArray( in_ctxt, ID_IN_ARRAY );		
					CDataArray2DVector3f outArray( in_ctxt );	
					for(CIndexSet::Iterator it = indexSet.Begin(); it.HasNext(); it.Next())
					{
						if(!inArray.GetCount())
							continue;
						CDataArray2DVector3f::Accessor inArrayA = inArray[it];
						CDataArray2DLong::Accessor inSizeA = inSizeArray[it];

						ULONG nbArray = inArrayA.GetCount();
						if(!nbArray)
							continue;

						ULONG nbGroupSizes = inSizeA.GetCount();
						ULONG group1st = 0, increment = inIncrement[it];
						ULONG nbGroups = nbGroupSizes > 1 ? nbGroupSizes : nbArray/inSizeA[0];

						CDataArray2DVector3f::Accessor outArrayA = outArray.Resize( it, nbGroups );
						for(int i_g=0; i_g<nbGroups; i_g++) 
						{
							ULONG groupSize = inSizeA[nbGroupSizes>1 ? i_g : 0];
							XSI::MATH::CVector3f xprod;
							if(groupSize) {							
								xprod = inArrayA[group1st];
								ULONG offset = 0;
								for(int i_ge=0; i_ge<groupSize; i_ge++) {
									offset += increment ? increment:1;
									if(group1st + offset < nbArray) 
										xprod.Cross(xprod, inArrayA[group1st+offset]);																
								}
							}
							outArrayA[i_g] = xprod;
							if(increment == 1)
								group1st += groupSize;
							else
								group1st += increment>1 ? 1:increment;
						}
					}
					break;
				}
				case XSI::siICENodeDataQuaternion:
				{
					CDataArray2DQuaternionf inArray( in_ctxt, ID_IN_ARRAY );	
					CDataArray2DQuaternionf outArray( in_ctxt );	
					for(CIndexSet::Iterator it = indexSet.Begin(); it.HasNext(); it.Next())
					{
						if(!inArray.GetCount())
							continue;
						CDataArray2DQuaternionf::Accessor inArrayA = inArray[it];
						CDataArray2DLong::Accessor inSizeA = inSizeArray[it];

						ULONG nbArray = inArrayA.GetCount();
						if(!nbArray)
							continue;

						ULONG nbGroupSizes = inSizeA.GetCount();
						ULONG group1st = 0, increment = inIncrement[it];
						ULONG nbGroups = nbGroupSizes > 1 ? nbGroupSizes : nbArray/inSizeA[0];

						CDataArray2DQuaternionf::Accessor outArrayA = outArray.Resize( it, nbGroups );
						for(int i_g=0; i_g<nbGroups; i_g++) 
						{
							ULONG groupSize = inSizeA[nbGroupSizes>1 ? i_g : 0];
							XSI::MATH::CQuaternionf xprod;							
							if(groupSize) {							
								xprod = inArrayA[group1st];
								ULONG offset = 0;
								for(int i_ge=0; i_ge<groupSize; i_ge++) {
									offset += increment ? increment:1;
									if(group1st + offset < nbArray) 
										xprod.Mul(xprod, inArrayA[group1st+offset]);																
								}
							}
							outArrayA[i_g] = xprod;
							if(increment == 1)
								group1st += groupSize;
							else
								group1st += increment>1 ? 1:increment;
						}
					}
					break;
				}
				case XSI::siICENodeDataColor4:
				{
					CDataArray2DColor4f inArray( in_ctxt, ID_IN_ARRAY );
					CDataArray2DColor4f outArray( in_ctxt );	
					for(CIndexSet::Iterator it = indexSet.Begin(); it.HasNext(); it.Next())
					{
						if(!inArray.GetCount())
							continue;
						CDataArray2DColor4f::Accessor inArrayA = inArray[it];
						CDataArray2DLong::Accessor inSizeA = inSizeArray[it];

						ULONG nbArray = inArrayA.GetCount();
						if(!nbArray)
							continue;

						ULONG nbGroupSizes = inSizeA.GetCount();
						ULONG group1st = 0, increment = inIncrement[it];
						ULONG nbGroups = nbGroupSizes > 1 ? nbGroupSizes : nbArray/inSizeA[0];

						CDataArray2DColor4f::Accessor outArrayA = outArray.Resize( it, nbGroups );
						for(int i_g=0; i_g<nbGroups; i_g++) 
						{
							ULONG groupSize = inSizeA[nbGroupSizes>1 ? i_g : 0];
							XSI::MATH::CColor4f xprod;
							if(groupSize) {							
								xprod = inArrayA[group1st];
								ULONG offset = 0;
								for(int i_ge=0; i_ge<groupSize; i_ge++) {
									offset += increment ? increment:1;
									if(group1st + offset < nbArray) 
										xprod.Mul(inArrayA[group1st+offset]);																
								}
							}
							outArrayA[i_g] = xprod;
							if(increment == 1)
								group1st += groupSize;
							else
								group1st += increment>1 ? 1:increment;
						}
					}
					break;
				}
	}

	return CStatus::OK;
}

